﻿using Microsoft.EntityFrameworkCore;
using Microsoft.Extensions.Configuration;
using Microsoft.Extensions.Logging;
using HealthMonitor.Core.Context;
using HealthMonitor.Core.DbLog;
using HealthMonitor.Core.Dal.EfCoreImpl;

namespace HealthMonitor.Core.Dal.Factory
{
	/// <summary>
	/// 瞬态GpsCardContext构建帮助类
	/// </summary>
	public class GpsCardAccessorFactory
	{
		private readonly IConfiguration _configuration;
		private readonly ILogger<GpsCardAccessorFactory> _logger;
		private readonly ILoggerFactory _loggerFactory;

		public GpsCardAccessorFactory(IConfiguration configuration, ILogger<GpsCardAccessorFactory> logger, IEfCoreLoggerProvider loggerProvider)
		{
			_configuration = configuration;
			_logger = logger;
			_loggerFactory = new LoggerFactory(new[] { loggerProvider });
		}

		public virtual IGpsCardDataAccessor GetDbContext()
		{
			var options = new DbContextOptionsBuilder<GpsCardContext>()
				.UseMySql(_configuration.GetConnectionString("GpsCard_Connection_String"))
				.UseLoggerFactory(_loggerFactory)
				.Options;

			var context = new GpsCardContext(options);
			return new EfCoreDataAccessor(context);

			////AOP需要依赖DI
			//var services = new ServiceCollection();
			//services.AddTransient<IGpsCardDataAccessor, EfCoreDataAccessor>(sp =>
			//{
			//	var context = new GpsCardContext(options);
			//	return new EfCoreDataAccessor(context);
			//});

			//var provider = services.BuildDynamicProxyProvider();
			//return provider.GetRequiredService<IGpsCardDataAccessor>();
		}
	}
}
